/**
 * Copyright 2021 MICRORISC s.r.o.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
#ifndef IQRF_HEADER_PARSER_H
#define IQRF_HEADER_PARSER_H

#include <iostream>
#include <regex>

/// IQRF header parser namespace
namespace iqrf_header_parser {

	/// IQRF plugin data line length
	static const uint8_t LINE_LENGTH = 40;

	/**
	 * OS version struct
	 */
	struct OsVersion {
		/// Major version
		uint8_t major;
		/// Minor version
		uint8_t minor;
		/// Build number
		uint16_t build;
	};

	/**
	 * Error code enum
	 */
	enum Error {
		NO_ERROR,
		UNSUPPORTED_UPLOAD_DEVICE = 4,
		INVALID_FILE_CONTENT,
		UNSUPPORTED_FILE_CONTENT,
		DEVICE_INCOMPATIBLE_FILE  
	};

	/**
	 * MCU type enum
	 */
	enum MCU {
		UNKNOWN,
		PIC16LF819,
		PIC16LF88,
		PIC16F886,
		PIC16LF1938,
		PIC16LF18877
	};

	/**
	 * TR series D enum
	 */
	enum TRD {
		TR_52D,
		TR_58D_RJ,
		TR_72D,
		TR_53D,
		TR_78D,
		TR_54D = 8,
		TR_55D,
		TR_56D,
		TR_76D,
		TR_77D,
		TR_75D
	};

	/**
	 * TR series G enum
	 */
	enum TRG {
		TR_72G = 2,
		TR_76G = 11
	};

	/**
	 * Validates and parses MCU header, stores error message in errorMsg
	 * @param line MCU and TR header
	 * @param mcu MCU type
	 * @param tr TR series
	 * @param errorMsg Error message container
	 * @return Error code
	 */
	Error parseMcuHeader(const std::string &line, const uint8_t &mcu, const uint8_t &tr, std::string &errorMsg);

	/**
	 * Validates and parses OS header, stores error message in errorMsg
	 * @param line OS header
	 * @param os Device OS information
	 * @param errorMsg Error message container
	 * @return Error code
	 */
	Error parseOsHeader(const std::string &line, const OsVersion &os, std::string &errorMsg);

	/**
	 * Validates DPA plugin header
	 * @param line Plugin header
	 * @return true if header is valid, false otherwise
	 */
	bool validPluginHeaderDpa(const std::string &line);

	/**
	 * Validates IQRF OS plugin header
	 * @param line Plugin header
	 * @return true if header is valid, false otherwise
	 */
	bool validPluginHeaderOs(const std::string &line);

	/**
	 * Validates and parses updateTime header, stores update time
	 *  in updateTime param if header is valid
	 * @param line Update time header
	 * @param updateTime Update time variable container
	 * @return true if header is valid, false otherwise
	 */
	bool parseUpdateHeader(const std::string &line, uint8_t &updateTime);

	/**
	 * Validates data line length and content, stores error message in errorMsg
	 * @param line Data line
	 * @param errorMsg Error message container
	 * @return true if line is valid, false otherwise
	 */
	bool validDataLine(const std::string &line, std::string &errorMsg);

	/**
	 * Checks if comment line is a separator
	 * @param line Comment line
	 * @return true if line is a separator, false otherwise
	 */
	bool isSeparator(const std::string &line);

	/**
	 * Splits string by delimiter and returns vector of substrings
	 * @param string String to be split
	 * @param delimiter Characters to split by
	 * @return String split by delimiter in vector
	 */
	std::vector<std::string> split(const std::string &string, const std::string &delimiter);

}
#endif
